#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <unistd.h>
#include <string.h>
#include "non_gui_defs.h"
#include "load_save.h"
#include "gui_defs.h"

/* write all the global variables into a file */
gint save_game(const gchar *file_name)
{
    gint i;
    FILE *fil = fopen(file_name, "wb");
    gchar new_file[400];
    
    if(fil == NULL)
	return -1;
    
    /* update game file name */
    strcpy(new_file, file_name);
    
    if(game_file != NULL)
	    g_free(game_file);

    namescan(&game_file, new_file);

    /* save dogs */
    for(i=0;i<420;i++)
	save_game_dog(dogs[i], fil);

    /* save kennels (well, only the names, really, as
       kennels consist only of the names) and qualified kennels */
    for(i=0;i<20;i++)
    {
	fwrite((void*)kennels[i].name, 14, 1, fil);
	if(i<6)
	    fwrite((void*)&qualified_kennels[i],
		   sizeof(gint), 1, fil);
    }

    /* save orders */
    for(i=0;i<420;i++)
    {
	fwrite((void*)&order[i], sizeof(gint), 1, fil);
	fwrite((void*)&statistics_order[0][i], sizeof(gint), 1, fil);
	fwrite((void*)&statistics_order[1][i], sizeof(gint), 1, fil);
    }
    fwrite((void*)&order[420], sizeof(gint), 1, fil);

    /*  read names !! */

    /* date */
    fwrite((void*)&season, sizeof(gint), 1, fil);
    fwrite((void*)&month, sizeof(gint), 1, fil);
    fwrite((void*)&week, sizeof(gint), 1, fil);
    
    save_game_races(fil);

    save_game_histories(fil);

    save_game_records(fil);

    save_game_options(fil);

    fclose(fil);

    return 0;
}

/* write the contents of a dog struct into a file */
gint save_game_dog(dog Dog, FILE *fil)
{
    gint i;
    
    /* first all the names and integers */
    fwrite((void*)&Dog.name, 19, 1, fil);
    fwrite((void*)&Dog.mother_name, 19, 1, fil);
    fwrite((void*)&Dog.father_name, 19, 1, fil);
    fwrite((void*)&Dog.id, sizeof(gint), 1, fil);
    fwrite((void*)&Dog.gender, sizeof(gint), 1, fil);
    fwrite((void*)&Dog.level, sizeof(gint), 1, fil);
    fwrite((void*)&Dog.age, sizeof(gint), 1, fil);
    fwrite((void*)&Dog.status, sizeof(gint), 1, fil);
    fwrite((void*)&Dog.mother_id, sizeof(gint), 1, fil);
    fwrite((void*)&Dog.father_id, sizeof(gint), 1, fil);
    
    for(i=0;i<99;i++)
    {
	fwrite((void*)&(Dog.speed[i]), sizeof(gint), 1, fil);
	fwrite((void*)&(Dog.stamina[i]), sizeof(gint), 1, fil);

	fwrite((void*)&(Dog.ancestor_speed[i]), sizeof(gint), 1, fil);
	fwrite((void*)&(Dog.ancestor_stamina[i]), sizeof(gint), 1, fil);
    }
    
    fwrite((void*)&Dog.fitness, sizeof(gfloat), 1, fil);
    fwrite((void*)&Dog.fertility, sizeof(gfloat), 1, fil);

    for(i=0;i<6;i++)
    {
	fwrite((void*)&(Dog.best_times[i]), sizeof(gfloat), 1, fil);
	fwrite((void*)&(Dog.races[i]), sizeof(gint), 1, fil);
    }

    for(i=0;i<10;i++)
	fwrite((void*)&(Dog.last_races[i]), sizeof(gfloat), 1, fil);

    return 0;
}

/* save the 12 races */
gint save_game_races(FILE *fil)
{
    gint i,j,k;
    gint no_dog = -1;
    
    for(i=0;i<2;i++)
	for(j=0;j<12;j++)
    {
	fwrite((void*)&(races[i][j].level), sizeof(gint), 1, fil);
	fwrite((void*)&(races[i][j].laps), sizeof(gint), 1, fil);
	
	for(k=0;k<10;k++)
	{
	    if(races[i][j].dogs[k] != NULL)
		fwrite((void*)&(races[i][j].dogs[k]->id),
		       sizeof(gint), 1, fil);
	    else
		fwrite((void*)&no_dog, sizeof(gint), 1, fil);

	    fwrite((void*)&(races[i][j].times[k]), sizeof(gfloat), 1, fil);
	    fwrite((void*)&(races[i][j].ranks[k]), sizeof(gint), 1, fil);
	}
    }
    
    return 0;
}

/* the histories are allocated in month 12 week 3 */
gint save_game_histories(FILE *fil)
{
    gint i;
    gint number_of_histories = (month != 12 || week != 3) ?
	season - 1 : season;
    history_element *element = first_history;

    fwrite((void*)&number_of_histories, sizeof(gint), 1, fil);

    for(i=0; i<number_of_histories; i++)
    {
	save_game_history_element(element, fil);
	element = element->next;
    }

    return 0;
}

gint save_game_history_element(history_element *element, FILE *fil)
{
    gint i;

    for(i=0;i<7;i++)
    {
	if(i<4)
	{
	    fwrite((void*)element->kennel_names[i], 14, 1, fil);
	    fwrite((void*)&(element->kennel_values[i]),
		   sizeof(gfloat), 1, fil);
	    fwrite((void*)&(element->user_kennel_results[i]),
		   sizeof(gint), 1, fil);
	    fwrite((void*)&(element->user_kennel_values[i]),
		   sizeof(gfloat), 1, fil);
	}

	if(i<3)
	{
	    fwrite((void*)element->user_dog_names[i], 19, 1, fil);
	    fwrite((void*)&(element->user_dog_values[i]),
		   sizeof(gfloat), 1, fil);
	}

	fwrite((void*)element->dog_names[i], 19, 1, fil);
	fwrite((void*)element->dog_kennel_names[i], 14, 1, fil);
	fwrite((void*)&(element->dog_values[i]), sizeof(gfloat), 1, fil);
    }
    
    return 0;
}

gint save_game_records(FILE *fil)
{
    gint i;

    for(i=0;i<7;i++)
    {
	fwrite((void*)dog_records[i].name, 19, 1, fil);
	fwrite((void*)dog_records[i].kennel_name, 14, 1, fil);
	fwrite((void*)&(dog_records[i].season), sizeof(gint), 1, fil);
	fwrite((void*)&(dog_records[i].value), sizeof(gfloat), 1, fil);

	if(i<5)
	{
	    fwrite((void*)kennel_records[i].name, 14, 1, fil);
	    fwrite((void*)&(kennel_records[i].season), sizeof(gint), 1, fil);
	    fwrite((void*)&(kennel_records[i].value), sizeof(gfloat), 1, fil);
	}
    }

    return 0;
}

gint save_game_options(FILE *fil)
{
    gint i;

    for(i=0;i<50;i++)
    {
	fwrite((void*)&options_misc[i], sizeof(gint), 1, fil);
	
	if(i<19)
	{
	    fwrite((void*)&(options_kennel_block[0][i]), sizeof(gint), 1, fil);
	    fwrite((void*)&(options_kennel_block[1][i]), sizeof(gint), 1, fil);
	}
	
	if(i<12)
	{
	    fwrite((void*)&options_show_news[i], sizeof(gint), 1, fil);
	    fwrite((void*)&options_confirm[i], sizeof(gint), 1, fil);
	}
    }

    return 0;
}

/* read all the global variables from a file.
   if before_game is 0, the game is loaded from
   within a game; else it's loaded at the beginning of the game, before
   even the 'ask_name' window appears */
gint load_game(gchar *file_name, gint before_game)
{
    gint i;
    dog test_dog;
    FILE *fil = fopen(file_name, "rb");
    
    load_game_dog(&test_dog, fil);
    
    /* a primitive check whether it's a 
       'greyhounds' save game */
    if(!(test_dog.id == 0 && test_dog.age > -2 && 
	 test_dog.age < 100))
	return 5;
    else
    {
	fclose(fil);
	fil = fopen(file_name, "rb");
    }

    /* load dogs */
    for(i=0;i<420;i++)
	load_game_dog(&dogs[i], fil);

    /* set game file name */
    if(game_file != NULL && before_game == 0)
	g_free(game_file);
    
    namescan(&game_file, file_name);

    /* load kennel names and qualified kennels */
    for(i=0;i<20;i++)
    {
	fread((void*)&(kennels[i].name), 14, 1, fil);
	if(i<6)
	    fread((void*)&qualified_kennels[i],
		   sizeof(gint), 1, fil);
    }

    /* load orders */
    for(i=0;i<420;i++)
    {
	fread((void*)&order[i], sizeof(gint), 1, fil);
	fread((void*)&statistics_order[0][i], sizeof(gint), 1, fil);
	fread((void*)&statistics_order[1][i], sizeof(gint), 1, fil);
    }
    fread((void*)&order[420], sizeof(gint), 1, fil);

    /* date */
    fread((void*)&season, sizeof(gint), 1, fil);
    fread((void*)&month, sizeof(gint), 1, fil);
    fread((void*)&week, sizeof(gint), 1, fil);
    
    load_game_races(fil);

    load_game_histories(fil);
    
    load_game_records(fil);

    load_game_options(fil);

    /* load the 500 dog names */
    if(load_dog_names() == 1)
	return 2;
    
    return 0;
}

/* read the contents of a dog struct from a file */
gint load_game_dog(dog *Dog, FILE *fil)
{
    gint i;
    
    /* first all the names and integers */
    fread((void*)&(Dog->name), 19, 1, fil);
    fread((void*)&(Dog->mother_name), 19, 1, fil);
    fread((void*)&(Dog->father_name), 19, 1, fil);
    fread((void*)&(Dog->id), sizeof(gint), 1, fil);
    fread((void*)&(Dog->gender), sizeof(gint), 1, fil);
    fread((void*)&(Dog->level), sizeof(gint), 1, fil);
    fread((void*)&(Dog->age), sizeof(gint), 1, fil);
    fread((void*)&(Dog->status), sizeof(gint), 1, fil);
    fread((void*)&(Dog->mother_id), sizeof(gint), 1, fil);
    fread((void*)&(Dog->father_id), sizeof(gint), 1, fil);
    
    for(i=0;i<99;i++)
    {
	fread((void*)&(Dog->speed[i]), sizeof(gint), 1, fil);
	fread((void*)&(Dog->stamina[i]), sizeof(gint), 1, fil);

	fread((void*)&(Dog->ancestor_speed[i]), sizeof(gint), 1, fil);
	fread((void*)&(Dog->ancestor_stamina[i]), sizeof(gint), 1, fil);
    }
    
    fread((void*)&(Dog->fitness), sizeof(gfloat), 1, fil);
    fread((void*)&(Dog->fertility), sizeof(gfloat), 1, fil);

    for(i=0;i<6;i++)
    {
	fread((void*)&(Dog->best_times[i]), sizeof(gfloat), 1, fil);
	fread((void*)&(Dog->races[i]), sizeof(gint), 1, fil);
    }

    for(i=0;i<10;i++)
	fread((void*)&(Dog->last_races[i]), sizeof(gfloat), 1, fil);
    
    return 0;
}

/* load the 12 races */
gint load_game_races(FILE *fil)
{
    gint i,j,k;
    gint no_dog;
    
    for(i=0;i<2;i++)
	for(j=0;j<12;j++)
    {
	fread((void*)&(races[i][j].level), sizeof(gint), 1, fil);
	fread((void*)&(races[i][j].laps), sizeof(gint), 1, fil);
	
	for(k=0;k<10;k++)
	{
	    fread((void*)&no_dog, sizeof(gint), 1, fil);

	    if(no_dog == -1)
		races[i][j].dogs[k] = NULL;
	    else
		races[i][j].dogs[k] = &dogs[no_dog];
	    
	    fread((void*)&(races[i][j].times[k]), sizeof(gfloat), 1, fil);
	    fread((void*)&(races[i][j].ranks[k]), sizeof(gint), 1, fil);
	}
    }
    
    return 0;
}

/* load histories */
gint load_game_histories(FILE *fil)
{
    gint i;
    gint number_of_histories;
    history_element *element = NULL;

    /* free old histories if necessary */
    free_histories();
    
    fread((void*)&number_of_histories, sizeof(gint), 1, fil);

    if(number_of_histories == 0)
    {
	first_history = NULL;
	return 1;
    }
    else if(number_of_histories == 1)
    {
	first_history = (history_element*)xmalloc(sizeof(history_element));
	first_history->next = NULL;
    }
    else
    {
	first_history = (history_element*)xmalloc(sizeof(history_element));
	first_history->next = NULL;
	
	element = first_history;
	
	for(i=0; i<number_of_histories - 1; i++)
	{	
	    element->next = (history_element*)xmalloc(sizeof(history_element));
	    element->next->next = NULL;
	    element = element->next;
	}
    }

    element = first_history;

    for(i=0; i<number_of_histories; i++)
    {
	load_game_history_element(element, fil);
	element = element->next;
    }

    return 0;
}

/* load one history element */
gint load_game_history_element(history_element *element, FILE *fil)
{
    gint i;

    for(i=0;i<7;i++)
    {
	if(i<4)
	{
	    fread((void*)&(element->kennel_names[i]), 14, 1, fil);
	    fread((void*)&(element->kennel_values[i]),
		   sizeof(gfloat), 1, fil);
	    fread((void*)&(element->user_kennel_results[i]),
		   sizeof(gint), 1, fil);
	    fread((void*)&(element->user_kennel_values[i]),
		   sizeof(gfloat), 1, fil);
	}

	if(i<3)
	{
	    fread((void*)&(element->user_dog_names[i]), 19, 1, fil);
	    fread((void*)&(element->user_dog_values[i]),
		   sizeof(gfloat), 1, fil);
	}

	fread((void*)&(element->dog_names[i]), 19, 1, fil);
	fread((void*)&(element->dog_kennel_names[i]), 14, 1, fil);
	fread((void*)&(element->dog_values[i]), sizeof(gfloat), 1, fil);
    }

    return 0;
}

gint load_dog_names(void)
{
    gint i, random;
    gint dog_names[500];
    gchar names[8224][19];
    gchar trash[2];
    FILE *fil=fopen(my_find_pixmap_file("names"),"r");

    if(fil == NULL)
	return 1;

    for(i=0;i<8224;i++)
    {
	if(i<500)
	    dog_names[i] = -1;
	fscanf(fil,"%[^\n]",names[i]);
	fscanf(fil,"%[\n]",trash);
    }
    fclose(fil);
    
    i=500;
    while(i > 0)
    {
	random = (gint)rint( rnd(0,8223) );
	while(is_in_list(dog_names, 0, 499, random) == 1)
	    random = (gint)rint( rnd(0,8223) );
	i--;
	strcpy(names_kept[i],names[random]);
	dog_names[i] = random;
    }

    return 0;
}

gint load_game_records(FILE *fil)
{
    gint i;

    for(i=0;i<7;i++)
    {
	fread((void*)&(dog_records[i].name), 19, 1, fil);
	fread((void*)&(dog_records[i].kennel_name), 14, 1, fil);
	fread((void*)&(dog_records[i].season), sizeof(gint), 1, fil);
	fread((void*)&(dog_records[i].value), sizeof(gfloat), 1, fil);

	if(i<5)
	{
	    fread((void*)&(kennel_records[i].name), 14, 1, fil);
	    fread((void*)&(kennel_records[i].season), sizeof(gint), 1, fil);
	    fread((void*)&(kennel_records[i].value), sizeof(gfloat), 1, fil);
	}
    }

    return 0;
}

gint load_game_options(FILE *fil)
{
    gint i;

    for(i=0;i<50;i++)
    {
	fread((void*)&options_misc[i], sizeof(gint), 1, fil);
	
	if(i<19)
	{
	    fread((void*)&(options_kennel_block[0][i]), sizeof(gint), 1, fil);
	    fread((void*)&(options_kennel_block[1][i]), sizeof(gint), 1, fil);
	}
	
	if(i<12)
	{
	    fread((void*)&options_show_news[i], sizeof(gint), 1, fil);
	    fread((void*)&options_confirm[i], sizeof(gint), 1, fil);
	}
    }

    return 0;
}

/* free the history list */
int free_histories(void)
{
    gint i, j;
    gint number_of_histories = 0;
    history_element *element = first_history;

    while(element != NULL)
    {
	element = element->next;
	number_of_histories++;
    }

    for(i=0;i<number_of_histories;i++)
    {
	element = first_history;
	for(j=0; j<number_of_histories - i - 1; j++)
	    element = element->next;
	    
	g_free(element);
    }
   
    return 0;
}
