#include<time.h>
#include<stdlib.h>
#include<stdio.h>
#include<math.h>
#include<string.h>
#include<unistd.h>
#include"non_gui_defs.h"

/* avoid segfaults with malloc */
void *xmalloc(size_t size)
{
    void *value = g_malloc (size);
    if(value == 0)
	g_print("Error: g_malloc returned a nul pointer.\n");
    
    return(value);
}

/* check whether 'item' is in array 'list' between
   'min' and 'max' */
gint is_in_list(gint *list, gint min, gint max, gint item)
{
    gint i;

    for(i=min;i<=max;i++)
	if(item == list[i])
	    return 1;
    
    return 0;
}

/* read an input up to 'maxlen' characters from the terminal 
   into the gchar array 'q' */
gint getinput(gchar *q,gint maxlen)
{
  gchar qtemp[30];

  scanf("%s",qtemp);
  
  if(strlen(qtemp) > maxlen)
    {
      strcpy(q,"-");
      return 1;
    }
  else
    strcpy(q,qtemp);
  
  return 0;
}

/* avoid segfaults with fopen */
FILE *ffopen(const gchar *file, const gchar *mode)
{
    gchar *path;
    FILE *fil;

    /* set global path of the text files needed
       for the program; also be the place where
       your saved games are written to; replace
       './' with the path where these files can be
       found if you'd like to copy the executable 'byghound'
       to /usr/bin or something */
    gchar *globpath=(char*)xmalloc(strlen("./"));    
    strcpy(globpath,"./");

    path=(char*)xmalloc(strlen(file) + strlen(globpath) + 1);

    strcpy(path,"\0");
    strcat(path,globpath);
    strcat(path,file);

    g_free(globpath);

    fil=fopen(path,mode);
 
    if(fil == NULL)
    {
	g_print("\n Could not open file: %s\n",path);
	sleep(1);
	g_free(path);
	if(strcmp(mode,"rb") != 0 && strcmp(mode,"wb") != 0)
	    exit(1);
	else
	    return NULL;
    }
    
    g_free(path);
    return fil;
}

/*  transform a value between 0 and 9.9 into an array
    of 0's and 1's that signifies part of the genetic
    code of a dog */
gint generate_code(gint *array, gfloat value)
{
    gint random,i;

    /* find out how many 1's there have to be in the array to
       fulfill the equation 'value = sum(array) / 10' */
    gint number_of_ones=(gint)rint( value * 10 );

    /* now set the appropriate number of elements with random
       indices to 1 or if it's faster to 0 */
    if(number_of_ones < 50)
    {
	for(i=0;i<99;i++)
	    array[i] = 0;
	while(number_of_ones > 0)
	{
	    random = (gint)rint( rnd(0,98) );
	    if( array[random] == 0 )
	    {
		array[random] = 1;
		number_of_ones--;
	    }
	}
    }
    else
    {
	number_of_ones = 99 - number_of_ones;
	for(i=0;i<99;i++)
	    array[i] = 1;
	while(number_of_ones > 0)
	{
	    random = (gint)rint( rnd(0,98) );
	    if( array[random] == 1 )
	    {
		array[random] = 0;
		number_of_ones--;
	    }
	}
    }

    return 0;
}

/* allocate the memory needed
   for 'dest' and copy 'source' into it */
gint namescan(gchar **dest, gchar *source)
{                                       
    *dest=(char*)xmalloc(strlen(source)+1);
    strcpy(*dest,source);

    return 0;
}

/* sort a gfloat array between indices 'first_element' and 'last_element';
   the function assumes that 'order' can hold
   'last_element' - 'first_element' + 1 entries */
gint sort_float_array(gfloat *array, gint *order, 
		     gint first_element, gint last_element)
{
    gint i, j, entries = last_element - first_element + 1;
    gint temporary_order[ last_element - first_element + 1];
    
    for(i=0;i<entries;i++)
	temporary_order[i] = 0;
    
    for(i=0;i<entries;i++)
	for(j=i+1;j<entries;j++)
	    if(array[first_element + i] > array[first_element + j])
		temporary_order[j]++;
	    else
		temporary_order[i]++;
    
    for(i=0;i<entries;i++)
    {
	for(j=0;j<entries;j++)
	    if(temporary_order[i] == j)
		order[j] = first_element + i;
    }    

    return 0;
}

/* allocate memory for 'dest' depending on 'size' and write the
   number 'number' into it */
gint number_to_char(gchar **dest, gfloat number, gint size)
{
    gchar temporary_char[6];
    
    *dest = (char*)xmalloc( size + 1 );
    
    /* if we should write the number as an integer */
    if(size < 6)
	sprintf(temporary_char, "%d", (gint)rint( number ));
    else
	sprintf(temporary_char, "%.2f",number);
 
    strcpy(*dest, temporary_char);

    return 0;
}

/* decompose an integer that stores multiple information
   just like the value you use whith 'chmod':
   chmod 4... means read only, 2 means write only, but
   6 = 4 + 2 means both; here we have the values 1, 2, 4 and 8.
   the function takes one of these and the compound value
   and returns 1 if it contains the single value and zero otherwise */
gint decompose_compound_integer(gint value, gint compound_value)
{
    if(value == compound_value)
	return 1;

    if(value == 1 && compound_value % 2 == 1)
	return 1;
    else if(value == 2)
    {
	if( (compound_value % 2 == 0 && compound_value % 4 != 0) ||
	    (compound_value % 2 == 1 && compound_value % 4 != 1) )
	    return 1;
    }
    else if(value == 4)
    {
	if( compound_value > 11 ||
	    (compound_value > 4 && compound_value < 8) )
	    return 1;
    }
    else if(value == 8 && compound_value > 7)
	return 1;

    return 0;
}

/* sleep for 'time' using 'sleep' and 'usleep' */
gint my_sleep(gfloat time)
{
    gint int_part = (gint)time;
    gfloat float_part = time - int_part;

    if(int_part > 0)
	sleep(int_part);

    if(float_part > 0)
	usleep(float_part);

    return 0;    
}
